C!!! NASA Goddard Space Flight Center Open Source: Jan McGarry (Jan.McGarry@nasa.gov)
C =======================================================================
C               * MATSOLV ROUTINES *
C               Jan McGarry/NASA/GSFC
C       MATRIX COMPUTATIONS AND LEAST SQUARES SOLUTIONS:
C          SOLVE    (LEAST SQUARES SOLUTION CALLING ROUTINE)
C          MATINV   (MATRIX INVERSION)
C          MATCOP   (MATRIX COPY)
C          LUDCMP   (LU DECOMPOSITION)
C          LUBKSB   (LU BACK SUBSTITUTION)
C          MATMUL   (MATRIX MULTIPLICATION)
C =======================================================================

This code was taken from Numerical Recipes and so is not included due
to licensing restrictions.  See website for information on getting
the routines listed above:  http://www.nr.com.


        SUBROUTINE SOLVE(A,B,N,IFLAG)
C -------------------------------------------------------------------
C Solve a set of simultaneous equations by LU decomposition and
C   back substituion.  See section 2.3 pp 31-38 of Numerical Recipes.
C   The equation to be solved (for X) is:  A*X = B
C
C   Inputs:  Matrix "A" is NxN.
C            Vector "B" is Nx1 and is the right-hand side of the equation.
C            "N" is the dimension of the system.
C
C   Outputs: "A" returns replaced with the LU decompostion.
C            "B" contains the solution "X" upon return.
C            "IFLAG" is "0" if solution is OK; "1" if solution failed.
C -------------------------------------------------------------------


        SUBROUTINE MATINV(A,N,AINV,IFLAG)
C -------------------------------------------------------------------
C Compute the inverse of NxN matrix "A".
C Inputs:  Matrix "A".
C          Dimension "N".
C
C Outputs: Matrix "AINV"
C          "IFLAG" is "0" if inversion was OK; "1" if NOT.
C
C Note: Input matrix is NOT destroyed.
C -------------------------------------------------------------------


        SUBROUTINE MATCOPY(A,B,N1,N2)
C ------------------------------------------------------------------------
C Copy matrix "A" over to matrix "B".  Both matrices have dimension N1xN2.
C ------------------------------------------------------------------------


        SUBROUTINE LUDCMP(A,N,INDX,D,IFLAG)
C ------------------------------------------------------------------------
C       * LU Decomposition of NxN input matrix "A" *
C       "A" is destroyed and replaced with its LU decomposition.
C       "INDX" is an output vector which records the row permutation
C              effected by partial pivoting.
C       "D" is output as +1 or -1 depending on whether the number of row
C              interchanges was even or odd.
C ------------------------------------------------------------------------


        SUBROUTINE LUBKSB(A,N,INDX,B)
C ------------------------------------------------------------------------
C       * LU Back Substitution *
C       Solves the set of "N" equations A*X=B.
C       "A" is input as the LU decomposition of the above equation,
C           with "INDX" as the LUDCMP output vector.
C       "B" is input as the right hand side of the equation, and returns
C              with the solution vector "X".
C ------------------------------------------------------------------------


      SUBROUTINE MATMUL(A,B,C,IM,IN,IP,IC,IS)
C from a routine by G.Kubizinsky/DBA
C************************************************************
C
C
C  IC = 1  R = A * B         IC = -1  RT = A * B
C  IC = 2  R = AT  * B       IC = -2  RT = AT * B
C  IC = 3  R = A * BT        IC = -3  RT = A * BT
C
C                       IS = -1        C = C - R
C                       IS =  0        C = R
C                       IS =  1        C = C + R
C
C        IM  =  NUMBER OF ROWS OF C
C        IP  =  NUMBER OF COLUMNS OF C
C        IN  =  THE COMMON DIMENSION OF A AND B
C
C    A,B,AND C  MUST BE UNIQUELY DEFINED
C
C************************************************************
